/**************************************************************

Craig Flory
CS307-01
Object Oriented Programming with C++
Instructor: Jagdish Bansiya
Fall Semester 1995

Date:    04 Dec 1995
Project: Language Interpreter

Class:    InputProc
Method:   Process()
Abstract: 
    The interface to prompt for, receive and process 
    user input. From here, appropriate Methods are called.

**************************************************************/

/*** INCLUDE FILES ***/

/* system */

#include <stdio.h>
#include <ctype.h>
#include <iostream.h>

/* local */

#include "cool.h"   
#include "inproc.h"
#include "parse.h"
#include "data.h"
#include "instruc.h"   


void InputProc::Process( int argc, char *argv[] )
{

int   
    status, first_flag;
char  
    str_buff [ MAX_STRING_LEN ],
    message  [ MAX_MESSAGE_LEN ];

class Parser      *parser;
class Data         data;
class Instruction *curr_ins;
class Instruction *prev_ins;
class Instruction *temp_ins;

InstructionID id;

FILE *fileptr;


/*==========*/
/*** INIT ***/
/*==========*/

parser = new Parser();

curr_ins = prev_ins = NULL;
first_flag = 0;    
fileptr = NULL;


/*=================*/
/*** INPUT FILE? ***/
/*=================*/

if( argc > 1 )
{
    // Read input from file specified on command line.

    fileptr = fopen( argv[ 1 ], "r" );
    if( fileptr == NULL )
    {
        sprintf( message, "\n%s%s", ERROR_OPEN_INFILE_STR, argv[1] );
        status = ERROR_OPEN_INFILE;
        goto RETURN;
    }
}


/*=============*/
/*** PROCESS ***/
/*=============*/

while( 1 ) // While reading program instructions...
{
    status = SUCCESS;
    str_buff[ 0 ] = message[ 0 ] = '\0';
    id = unassigned;

    status = 
        GetStringInput( fileptr, str_buff );
    if( status != SUCCESS )
    {
        if( status == WARN_INPUT_EMPTY )
            continue; // allow blank lines

        GetMessage( message );
        goto RETURN;
    }           
 
printf("Entry was %s", str_buff ); 


    // Get an Instruction object, based on input string.

    curr_ins = parser->GetInstruc( str_buff );

    if( curr_ins == NULL )
    {
        // Error getting instruction
        parser->GetMessage( message );
        status = ERROR_INVAL_INSTR;
        goto RETURN;
    }

    if( first_instruc == NULL )
    {
        // Set the first instruction in the List.

        SetFirstInstruc( curr_ins );
        prev_ins = curr_ins;
        first_flag = 1;    
    }
    else first_flag = 0; 
    

    id = curr_ins->GetInstrucID();


    if( GetNumPtrs() > 0 )
    {  
        // We currently have instruction pointer(s) on the stack.
        // These are either Loops or IfThen conditionals. 
        // These types of instructions keep their own linked
        // lists of instructions. Pop a ptr - GetInstrucPtr() -  
        // and add the current instruction to the ptr's instruction set.

        status = GetInstrucPtr( &temp_ins );
        if( status != SUCCESS ) goto RETURN;

        status = temp_ins->AddInstruction( curr_ins );
        if( status != SUCCESS )
        {   
            temp_ins->GetMessage( message );
            goto RETURN;
        }

        if( id != endwhile && id != endconditional )
        {
            // Not at the end of a instruction set.
            // Place instruction ptr back on stack.

            status = AddInstrucPtr( temp_ins );
            if( status != SUCCESS ) goto RETURN;
        }
            
    } /*** END IF: processing the instruction STACK ***/

    else if( !first_flag )
    {
        // This is where the linked list of instructions is 
        // maintained. For each new instruction, add it to
        // the "next" pointer of the previous instruction.

        prev_ins->SetNext( curr_ins );
        prev_ins = curr_ins; 
    }
  

    if( id == whilene || id == ifthen )
    {
        // Building a loop OR conditional set of instructions. 
        // Add current instruction ptr to stack so its used next.

        status = AddInstrucPtr( curr_ins );
        if( status != SUCCESS ) goto RETURN;

        // Add to program instruction set

        //prev_ins->SetNext( curr_ins );
        //prev_ins = curr_ins; 

        //continue;
    }
    else if( id == end ) 
        break; // Program End.


} /*** END WHILE (1) ***/


// EXECUTE the instruction set 

status = SUCCESS;
curr_ins = GetFirstInstruc();

while( curr_ins != NULL )
{
    status = curr_ins->Execute( data );
    if( status != SUCCESS ) 
    {   
        curr_ins->GetMessage( message );
        goto RETURN;
    }

    temp_ins = curr_ins->GetNext();
    delete curr_ins;
    curr_ins = temp_ins;
}


RETURN:


if( status != SUCCESS )
{
    cout << message << endl << endl;
    cout << flush;
}


return;
}
  
