/***************************************************************

Craig Flory
CS307-01
Object Oriented Programming with C++
Instructor: Jagdish Bansiya
Fall Semester 1995

Date:    04 Dec 1995
Project: Language Interpreter

File: inproc.h
Abstact:
    Contains each class definition and any other defines
    required by the Input Processing components.

****************************************************************/

#ifndef INPROC_INC_H
#define INPROC_INC_H

#include <stdio.h>
#include <string.h>

#include "cool.h"
#include "instruc.h"

 
/***************************/
/***  CLASS DEFINITIONS  ***/
/***************************/
  

///////////////////////////////////////////////////////////////////
//  CLASS - STACK NODE  //
//////////////////////////

class StackNode
{
private:
    class StackNode   *next;  // Link to other nodes in Stack
    class Instruction *item;  // Data item in the stack

public: 

    ////////////////////////////////
    // Constructor & Destructor
            
    StackNode ( void ) { next = NULL; item = NULL; }
    ~StackNode( void ) { ; }  

    ////////////////////////////////
    // Member/Access Functions
   
    void  SetDataItem( class Instruction *iptr ) { item = iptr; }   
    class Instruction * GetDataItem( void ) { return( item ); }

    StackNode * GetNextPtr( void ) { return next; }
    void SetNextPtr( StackNode *inptr ) { next = inptr; }
};


///////////////////////////////////////////////////////////////////
//  CLASS - STACK  //
/////////////////////

class Stack
{
private:
    int   num_nodes;        // Num nodes in Stack
    class StackNode *list;  // Link to nodes in Stack
    char  message[ MAX_MESSAGE_LEN ];

public: 

    ////////////////////////////////
    // Constructor & Destructor
            
    Stack ( void ) 
    { 
        num_nodes = 0; 
        list = NULL;
        message[0] = '\0';    
    }
    ~Stack( void );

    ////////////////////////////////
    // Member/Access Functions

    int  Push ( class Instruction *iptr );   
    int  Pop  ( class Instruction **iptr ); 
    int  Full ( void ) { return( 0 ); } // always notfull
    int  Empty( void );

    int  GetNumNodes( void ) { return( num_nodes); }
    inline void GetMessage( char msg[] ) { strcpy( msg, message ); }    
};

    
////////////////////////////////////////////////////////////////
//  CLASS - InputProc   
//  Abstract:
//      This class is the utility which reads lines of
//      input, dispatches parsing, and finally initiates 
//      COOL program execution 
////////////////////////////////////////////////////////////////
 
class InputProc
{
private:    

    char message[ MAX_MESSAGE_LEN ];  // (Error) Message returned by 
                                      // methods.
    class Stack *instruc_stack;
    class Instruction *first_instruc;

public:

    ////////////////////////////////
    // Constructor & Destructor

    InputProc ( void );
    ~InputProc( void ) 
    {
        if( instruc_stack ) delete instruc_stack;
    }

     ///////////////////////////////
    // Member Functions

    void  Process( int argc, char *argv[] );
    int   GetStringInput( FILE *fileptr, char *output_str );
    int   ValidateInput ( char *input_str  );

    int   AddInstrucPtr( class Instruction *iptr );
    int   GetInstrucPtr( class Instruction **iptr );
    int   GetNumPtrs( void ) { return(instruc_stack->GetNumNodes()); }

    void  SetFirstInstruc( class Instruction *iptr )
    {
        first_instruc = iptr;
    }
    class Instruction * GetFirstInstruc( void )
    {
        return( first_instruc );
    }

    inline void GetMessage( char msg[] ) { strcpy( msg, message ); }
};


#endif


