
/***************************************************************

Craig Flory
CS307-01
Object Oriented Programming with C++
Instructor: Jagdish Bansiya
Fall Semester 1995

Date:    04 Dec 1995
Project: Language Interpreter

File: instruc.h
Abstract: 
    Contains each class definition and any other defines
    required by the Instruction Base Class.

****************************************************************/

#ifndef INSTRUC_INC_H
#define INSTRUC_INC_H

/**********************/
/***  HEADER FILES  ***/
/**********************/

/* system */

#include <stdio.h>
#include <string.h>

#include <iostream.h>

/* local */

#include "cool.h"
#include "data.h"


/*****************/
/***  DEFINES  ***/
/*****************/

#define INSTR_END_HEADING_STR   "Referenced Variable Values:"
#define INSTR_END_DELIMIT_STR   "------------------------------"


/*****************/
/***  CLASSES  ***/
/*****************/

////////////////////////////////////////////////////////////////
//  ABSTRACT CLASS - Instruction  
//  Abstract: Class describing an Instruction Base Class 
////////////////////////////////////////////////////////////////

class Instruction
{
private:

    // Properties/Attributes

    InstructionID ID;  /* ID of instruction (enum) */

protected:
 
    char  message[ MAX_MESSAGE_LEN ];

public:

    class Instruction *next;

    //////////////////////////
    // Constructor & Destructor

    Instruction ( void );
    virtual ~Instruction( void ) {;}

    /////////////////////////
    // Overloaded Functions

    /////////////////////////
    // Member Functions

    int GetDataIndex( char *in_str, int &index  );
    int GetDataInt  ( char *in_str, long &value );

    // Pure Virtual Functions

    virtual int  Execute  ( class Data &data ) = 0;
    virtual int  SetParams( char *params[]   ) = 0;
    virtual int  AddInstruction( class Instruction *iptr ) = 0;

    /////////////////////////
    // Access Functions

    class Instruction * GetNext( void ) { return (next); }
    void SetNext( class Instruction *iptr ) { next = iptr; }

    InstructionID GetInstrucID( void )    { return (ID); }
    void SetInstrucID( InstructionID id ) { ID = id; }

    inline void GetMessage( char msg[] ) { strcpy( msg, message ); }

};


/*=================*/
/*** SUB-CLASSES ***/
/*=================*/

////////////////////////////////////////////////////////////////
//  ABSTRACT BASE CLASS - Instruction
//  ABSTRACT CLASS - Sequential 
////////////////////////////////////////////////////////////////


class Sequential : public Instruction
{
private:

    // Properties/Attributes

    // Member Functions

public:

    //////////////////////////////////
    // Constructor & Destructor

    // Chain constuctor
     Sequential( void ) : Instruction(){;}  
    ~Sequential( void ) {;}

    //////////////////////////////////
    // Shared Pure Virtual Functions

    int  Execute  ( class Data &data ) = 0;
    int  SetParams( char *params[]   ) = 0;
    int  AddInstruction( class Instruction *iptr ) = 0;

    /////////////////////////
    // Overloaded Functions

    /////////////////////////
    // Access Functions

};


////////////////////////////////////////////////////////////////
//  BASE CLASS - Instruction
//  CLASS      - Loop  
////////////////////////////////////////////////////////////////

class Loop : public Instruction
{
private:


protected:

    // Properties/Attributes
       
    /* flag to say we've been issued an endloop instruction */

    int   end_flag; 

    /* instructions that make up the loop */

    class Instruction *start;
    class Instruction *end;

    // Member Functions

public:

    //////////////////////////
    // Constructor & Destructor

    // Chain constuctor
     Loop( void ) : Instruction() { start = end = NULL; end_flag = 0; }
    ~Loop( void ) {;}


    /////////////////////////
    // Overloaded Functions


    //////////////////////////////////
    // Shared Pure Virtual Functions

    int  Execute  ( class Data &data ) = 0;
    int  SetParams( char *params[]   ) = 0;
    int  AddInstruction( class Instruction *iptr ) = 0;

    /////////////////////////
    // Access Functions

    class Instruction * GetStart( void ) { return (start); }

};


////////////////////////////////////////////////////////////////
//  ABSTRACT BASE CLASS - Instruction
//  ABSTRACT CLASS      - Conditional  
////////////////////////////////////////////////////////////////

class Conditional : public Instruction
{
private:

    // Properties/Attributes

    // Member Functions
   
protected:

    int   end_flag;
    int   else_flag;

    class Instruction *if_condition;
    class Instruction *else_condition;

    class Instruction *if_condition_end;
    class Instruction *else_condition_end;

public:

    //////////////////////////
    // Constructor & Destructor

    // Chain constructor 
     Conditional( void ) : Instruction() 
     {
        if_condition = else_condition = NULL;
        if_condition_end = else_condition_end = NULL;
        end_flag = else_flag = 0;
     } 
    ~Conditional( void ) {;}

    ///////////////////////////////////
    // Shared Pure Virtual Functions

    int  Execute  ( class Data &data ) = 0;
    int  SetParams( char *params[]   ) = 0;
    int  AddInstruction( class Instruction *iptr ) = 0;

    /////////////////////////
    // Overloaded Functions

    /////////////////////////
    // Access Functions

};



/*=====================*/
/*** SUB-SUB CLASSES ***/
/*=====================*/

////////////////////////////////////////////////////////////////
//  BASE CLASS - Instruction
//  SUB CLASS  - Sequential
//  CLASS      - Assign
//  Abstract: 
//      Assign integer long1 to variable at index1. 
///////////////////////////////////////////////////////////////

class Assign : public Sequential
{
private:

    // Properties/Attributes
   
    long  long1;
    int   index1;
    int   index2;

    // Member Functions

public:

    /////////////////////////////
    // Constructor & Destructor

    // Chain constructor
    Assign( void ) : Sequential()
    { 
        long1 = 0; 
        index1 = index2 = COOL_INDEX_UNASSIGNED; 
    }
    ~Assign( void ) {;}


    /////////////////////////////
    // Shared Virtual Functions
                                       
    int  Execute  ( class Data &data ); 
    int  SetParams( char *params[]   );
    int  AddInstruction( class Instruction *iptr ) { return(SUCCESS); } 


    /////////////////////////
    // Access Functions

};


////////////////////////////////////////////////////////////////
//  BASE CLASS - Instruction
//  SUB CLASS  - Sequential
//  CLASS      - Increment
//  Abstract: 
//      Increment by 1, the variable at index1. 
///////////////////////////////////////////////////////////////

class Increment : public Sequential
{
private:

    // Properties/Attributes

    int index1;

    // Member Functions

public:

    //////////////////////////
    // Constructor & Destructor

    // Chain constructor 
    Increment( void ) : Sequential() 
    { 
        index1 = COOL_INDEX_UNASSIGNED; 
    }
    ~Increment( void ) {;}

    /////////////////////////
    // Shared Virtual Functions

    int Execute( class Data &data ) 
    { 
        data[index1]++; 
        return(SUCCESS);
    }
    int  SetParams( char *params[]   );
    int  AddInstruction( class Instruction *iptr ) { return(SUCCESS); } 

    /////////////////////////
    // Access Functions

};


////////////////////////////////////////////////////////////////
//  BASE CLASS - Instruction
//  SUB CLASS  - Sequential
//  CLASS      - Add
//  Abstract: 
//      Add integer long1 to the variable at index1. 
///////////////////////////////////////////////////////////////

class Add : public Sequential
{
private:

    // Properties/Attributes
   
    long  long1;
    int   index1;
    int   index2;

public:

    //////////////////////////
    // Constructor & Destructor

    // Chain constructor
    Add( void ) : Sequential()
    { 
        long1 = 0; 
        index1 = index2 = COOL_INDEX_UNASSIGNED; 
    }
    ~Add( void ) {;}

    //////////////////////////////
    // Shared Virtual Functions

    int  Execute  ( class Data &data ); 
    int  SetParams( char *params[]   ); 
    int  AddInstruction( class Instruction *iptr ) { return(SUCCESS); }

};


////////////////////////////////////////////////////////////////
//  BASE CLASS - Instruction
//  SUB CLASS  - Sequential
//  CLASS      - Subtract
//  Abstract: 
//      Subtract integer long1 from the variable at index1. 
///////////////////////////////////////////////////////////////

class Subtract : public Sequential
{
private:

    // Properties/Attributes
   
    long long1;
    int  index1;
    int  index2;

public:

    //////////////////////////
    // Constructor & Destructor

    // Chain constructor
    Subtract( void ) : Sequential() 
    { 
        long1 = 0; 
        index1 = index2 = COOL_INDEX_UNASSIGNED; 
    }
    ~Subtract( void ) {;}

    /////////////////////////
    // Shared Virtual Functions

    int  Execute  ( class Data &data ); 
    int  SetParams( char *params[]   );
    int  AddInstruction( class Instruction *iptr ) { return(SUCCESS); } 

};


////////////////////////////////////////////////////////////////
//  BASE CLASS - Instruction
//  SUB CLASS  - Sequential
//  CLASS      - Multiply
//  Abstract: 
//      Multiply integers long1 & long2 and assign to the 
//      variable at index1. 
///////////////////////////////////////////////////////////////

class Multiply : public Sequential
{
private:

    // Properties/Attributes
   
    long  long1;
    long  long2;
    int   index1;

public:

    //////////////////////////
    // Constructor & Destructor

    // Chain constructor
    Multiply( void ) : Sequential()
    { 
        long1 = long2 = 0; 
        index1 = COOL_INDEX_UNASSIGNED; 
    }
    ~Multiply( void ) {;}

    /////////////////////////
    // Shared Virtual Functions

    int Execute( class Data &data ) 
    { 
        data[index1] = long1 * long2;
        return(SUCCESS);
    }
    int  SetParams( char *params[] ); 
    int  AddInstruction( class Instruction *iptr ) { return(SUCCESS); }

};


////////////////////////////////////////////////////////////////
//  BASE CLASS - Instruction
//  SUB CLASS  - Sequential
//  CLASS      - Copy
//  Abstract: 
//      Copy the variable value at index1, to the variable 
//      at index2. 
///////////////////////////////////////////////////////////////

class Copy : public Sequential
{
private:

    // Properties/Attributes
   
    int index1;
    int index2;

public:

    //////////////////////////
    // Constructor & Destructor

    // Chain constructor
    Copy( void ) : Sequential()
    { 
        index1 = index2 = COOL_INDEX_UNASSIGNED; 
    }
    ~Copy( void ) {;}

    /////////////////////////
    // Shared Virtual Functions

    int Execute( class Data &data ) 
    { 
        data[index1] = data[index2]; 
        return(SUCCESS);
    }
    int  SetParams( char *params[] );
    int  AddInstruction( class Instruction *iptr ) { return(SUCCESS); } 

};


////////////////////////////////////////////////////////////////
//  BASE CLASS: Instruction
//  SUB CLASS : Sequential
//  CLASS     : EndSet
//  Abstract  : An End-of marker 
///////////////////////////////////////////////////////////////

class EndSet : public Sequential
{
private:

    // Properties/Attributes

    // Member Functions

public:

    //////////////////////////
    // Constructor & Destructor

    // Chain constructor
     EndSet( void ) : Sequential() {;}
    ~EndSet( void ) {;}

    //////////////////////////////////////////////
    // Shared Virtual Functions - not implemented

    int  Execute  ( Data &data     ) { return(SUCCESS); }
    int  SetParams( char *params[] ) { return(SUCCESS); }
    int  AddInstruction( class Instruction *iptr ) { return(SUCCESS); }
};


////////////////////////////////////////////////////////////////
//  BASE CLASS - Instruction
//  SUB CLASS  - Sequential
//  CLASS      - End
//  Abstract: 
//      Marks the end of the program. Output all variable vals. 
///////////////////////////////////////////////////////////////

class End : public Sequential
{
private:

public:

    //////////////////////////
    // Constructor & Destructor

     End( void ) : Sequential() {;}
    ~End( void ) {;}

    /////////////////////////
    // Shared Virtual Functions

    /* print all variable values */
    int  Execute  ( class Data &data );
    int  SetParams( char *params[] )   { return(SUCCESS); }
    int  AddInstruction( class Instruction *iptr ) { return(SUCCESS); } 

};


////////////////////////////////////////////////////////////////
//  BASE CLASS - Instruction
//  SUB CLASS  - Sequential
//  CLASS      - End
//  Abstract   - Marks the beginning of an ELSE clause.
///////////////////////////////////////////////////////////////

class Else : public Sequential
{
private:

public:

    //////////////////////////
    // Constructor & Destructor

     Else( void ) : Sequential() {;}
    ~Else( void ) {;}

    /////////////////////////
    // Shared Virtual Functions

    /* print all variable values */
    int  Execute  ( class Data &data ) { return(SUCCESS); }
    int  SetParams( char *params[] )   { return(SUCCESS); }
    int  AddInstruction( class Instruction *iptr ) { return(SUCCESS); } 

};


////////////////////////////////////////////////////////////////
//  BASE CLASS - Instruction
//  SUB CLASS  - Loop
//  CLASS      - LoopWhile 
//  Abstract:
//      while var1 != var2, execute loop's instructions
///////////////////////////////////////////////////////////////

class LoopWhile : public Loop
{
private:

    // Properties/Attributes
   
    int  index1;
    int  index2;

public:

    //////////////////////////
    // Constructor & Destructor

    // Chain constructor
    LoopWhile( void ) : Loop() 
    { 
        index1 = index2 = COOL_INDEX_UNASSIGNED;
    }
    ~LoopWhile( void ) {;}

    /////////////////////////
    // Shared Virtual Functions

    int  Execute  ( class Data &data );
    int  SetParams( char *params[]   ); 
    int  AddInstruction( class Instruction *iptr );

};


////////////////////////////////////////////////////////////////
//  BASE CLASS - Instruction
//  SUB CLASS  - Conditional
//  CLASS      - IfThenElse
//  Abstract:
//      if( var@index1 != 0 ) execute instruc_set1
//      else execute instruc_set2
///////////////////////////////////////////////////////////////

class IfThenElse : public Conditional
{
private:

    // Properties/Attributes
   
    int index1;

public:

    //////////////////////////
    // Constructor & Destructor

    // Chain constructor
    IfThenElse( void ) : Conditional() 
    { 
        index1 = COOL_INDEX_UNASSIGNED;
    }
    ~IfThenElse( void ) {;}

    ///////////////////////////////
    // Shared Virtual Functions

    int  Execute  ( class Data &data );
    int  SetParams( char *params[] ); 
    int  AddInstruction( class Instruction *iptr );

};


#endif
