
/***************************************************************

Craig Flory
CS307-01
Object Oriented Programming with C++
Instructor: Jagdish Bansiya
Fall Semester 1995

Date:    04 Dec 1995
Project: Language Interpreter

File: cool.h
Abstract: 
    Contains defines and common data required by the 
    Language Interpreter classes.

****************************************************************/

#ifndef COOL_INC_H
#define COOL_INC_H


/*****************/
/***  DEFINES  ***/
/*****************/

/*** GLOBAL TO ALL PROCESSES ***/

// Return Status of Methods

#define SUCCESS                  0   
#define ERROR_SEVERITY_LOW       1
#define ERROR_ALLOC              13
#define ERROR_ALLOC_STR          "Error: allocating memory"

// Processing Messages and return values

#define ERROR_OPEN_INFILE               18
#define ERROR_OPEN_INFILE_STR           "Error: Opening input file: "

#define ERROR_INVAL_INSTR               20
#define ERROR_INVAL_INSTR_STR           "Error: Invalid Instruction: "
#define ERROR_INVAL_INPUT               21
#define ERROR_INVAL_INPUT_STR           "Invalid input" 

#define ERROR_STACK_FULL                30
#define ERROR_STACK_FULL_STR            "Error: Stack is full"
#define ERROR_STACK_EMPTY               31
#define ERROR_STACK_EMPTY_STR           "Error: Stack is empty"

#define ERROR_DATA_INDEX_INVAL          36
#define ERROR_DATA_INDEX_INVAL_STR      "Invalid data index: "
#define ERROR_DATA_VAR                  37
#define ERROR_DATA_VAR_STR              "Invalid data variable: "
#define ERROR_DATA_VAL_INT              38
#define ERROR_DATA_VAL_INT_STR          "Invalid data integer value: "

#define ERROR_LOOP_NO_END               40
#define ERROR_LOOP_NO_END_STR           "Error: Missing end-of-loop instruction"
#define ERROR_LOOP_END_ADDINS           41
#define ERROR_LOOP_END_ADDINS_STR       "Error: Trying to add instruction after endloop"

#define ERROR_COND_NO_END               45
#define ERROR_COND_NO_END_STR           "Error: Missing end-of-if instruction"
#define ERROR_COND_END_ADDINS           46
#define ERROR_COND_END_ADDINS_STR       "Error: Trying to add instruction after endif"

#define WARN_INPUT_EMPTY                100
#define WARN_INPUT_EMPTY_STR            "Input is empty"
#define WARN_EOF                        101
#define WARN_EOF_STR                    "EOF reached before \"end\" or error occured in file"


// Other

#define MAX_MESSAGE_LEN          128   // Max num chars in a message.
#define MAX_STRING_LEN           128   // Max num chars expected in a string.  


/*** Data Info ***/

#define COOL_MAX_PARAMS               3  // Max params in an instruction
#define COOL_MAX_VARS_INT            10  // Max num integer variables
#define COOL_MAX_INSTR_LEN            8  // Max strlen of a COOL instruc

#define COOL_VAR_A                   'A'  // Input chars for variable
#define COOL_VAR_B                   'B'  
#define COOL_VAR_C                   'C' 
#define COOL_VAR_D                   'D'  
#define COOL_VAR_E                   'E'  
#define COOL_VAR_F                   'F'  
#define COOL_VAR_G                   'G'  
#define COOL_VAR_H                   'H'  
#define COOL_VAR_I                   'I'  
#define COOL_VAR_J                   'J'  

#define COOL_INIT_VAL_INT             0    // Init integer value
#define COOL_INDEX_UNASSIGNED         -99

/*** Instruction Info ***/

/* strings */

#define COOL_INSTR_ASSIGN_STR           "ASSIGN"
#define COOL_INSTR_INC_STR              "INC"
#define COOL_INSTR_ADD_STR              "ADD"
#define COOL_INSTR_SUB_STR              "SUB"
#define COOL_INSTR_MULTIPLY_STR         "MUL"
#define COOL_INSTR_COPY_STR             "COPY"
#define COOL_INSTR_END_STR              "END"
#define COOL_INSTR_WHILENE_STR          "WHILENE"
#define COOL_INSTR_ENDWHILE_STR         "ENDWHILE"
#define COOL_INSTR_IFTHEN_STR           "IFTHEN"
#define COOL_INSTR_ELSE_STR             "ELSE"
#define COOL_INSTR_ENDIF_STR            "ENDIF"


/********************/
/***  STRUCTURES  ***/
/********************/

/* Instruction Info */

typedef enum 
{ assign, increment, add, subtract, multiply, copy, 
  whilene, endwhile, ifthen, ifelse, endconditional, end, unassigned 
} InstructionID;


#endif
