/***************************************************************

Craig Flory
CS307-01
Object Oriented Programming with C++
Instructor: Jagdish Bansiya
Fall Semester 1995

Date:    04 Dec 1995
Project: Language Interpreter

Base Class:  Instruction
Sub Class :  Conditional 
Class     :  IfThenElse
Member    :  AddInstruction()
Abstract  :  
    Add an instruction to the one of the condition 
    (if or else) instruction sets. IF the "else_flag"
    is NOT set, it means we're in an IF condition and
    instructions should be added there; ELSE add them
    to the ELSE condition instruction set.
    
Notes:
    WARNING! If an "endconditional" instruction is passed, it
    is used only to set our end_flag. It is then deleted!
    That way, the memory created by the calling function
    is freed since its not used by anything else.

References:
(1) Weiskamp, Keith & Flamig Bryan, "The Complete C++ Primer"
(2) Standish, Thomas, "Data Structures, Algorithms, and Software
    Principles in C"

*****************************************************************/


/*** INCLUDE FILES ***/

#include <stdio.h>

#include "instruc.h"


int IfThenElse::AddInstruction( class Instruction *instruc )
{
    

InstructionID id;

    id = instruc->GetInstrucID();


    // Set flags as applicable

    if( id == endconditional || id == ifthen || id == ifelse )
    {
        // These instructions are only used to set flags
        // in the object. Set our flag and leave.

        if( id == endconditional )
            end_flag = 1;   // correct input syntax
    
        else if( id == ifelse )
            else_flag = 1;  // begin ELSE condition

        else if( id == ifthen )
            else_flag = end_flag = 0;  // reset flags

        delete instruc;
        instruc = NULL;
        return( SUCCESS );
    }


    if( end_flag )
    {
        // Naughty. Trying to add instruction after the 
        // conditional has been officially ended.

        strcpy( message, ERROR_COND_END_ADDINS_STR );
        return( ERROR_COND_END_ADDINS );
    }

    // Add instruction as applicable

    if( !else_flag )
    {
        if( if_condition == NULL )
        {
            if_condition = if_condition_end = instruc;
        }
        else
        {
            // Link input to the current end of the list
            // of instructions, and then reset end.

            if_condition_end->next = instruc;
            if_condition_end = instruc;
        }  
    }
    else
    {
        if( else_condition == NULL )
        {
            else_condition = else_condition_end = instruc;
        }
        else
        {
            // Link input to the current end of the list
            // of instructions, and then reset end.

            else_condition_end->next = instruc;
            else_condition_end = instruc;
        }  
    }


return( SUCCESS );
}
