/***************************************************************

Craig Flory
CS307-01
Object Oriented Programming with C++
Instructor: Jagdish Bansiya
Fall Semester 1995

Date:    04 Dec 1995
Project: Language Interpreter

Base Class:  Instruction
Sub Class :  Sequential 
Class     :  LoopWhile
Member    :  AddInstruction()
Abstract  :  
    Add an instruction to the loop list of instructions.
    
Notes:
    WARNING! If an "endwhile" instruction is passed, it
    is used only to set our end_flag. It is then deleted!
    That way, the memory created by the calling function
    is freed.

References:
(1) Weiskamp, Keith & Flamig Bryan, "The Complete C++ Primer"
(2) Standish, Thomas, "Data Structures, Algorithms, and Software
    Principles in C"

*****************************************************************/


/*** INCLUDE FILES ***/

#include <stdio.h>

#include "instruc.h"


int LoopWhile::AddInstruction( class Instruction *instruc )
{
    
    if( (instruc->GetInstrucID()) == endwhile )
    {
        // This instruction is only used to mark the end of
        // the loop. Set our flag (signifying correct syntax)
        // and leave. Free the unused instruction.

        end_flag = 1;
        delete instruc;
        instruc = NULL;
        return( SUCCESS );
    }

    
    if( end_flag )
    {
        // Naughty. Trying to add instruction after the loop
        // has been officially ended.

        strcpy( message, ERROR_LOOP_END_ADDINS_STR );
        return( ERROR_LOOP_END_ADDINS );
    }


    if( start == NULL )
    {
        start = end = instruc;
    }
    else
    {
        // Link input to the current end of the list
        // of loop instructions, and then reset end.

        end->next = instruc;
        end = instruc;
    }


return( SUCCESS );
}
