
/**************************************************************

Craig Flory
CS307-01
Object Oriented Programming with C++
Instructor: Jagdish Bansiya
Fall Semester 1995

Date:    04 Dec 1995
Project: Language Interpreter

Class:    Parser
Method:   GetInstruc()
Abstract: 
    This function will completely parse the input string,
    assigning the instruction and its parameters in the
    return Instruction object.

Assumptions: Input string is not empty

Notes:
    Only the minimum components of an input string are
    parsed. If anything beyond the minimum is used, it
    will be ignored. For example, a valid Increment 
    instruction might be  INC(A), but  INC(A,3,4,) 
    would also give the same results since only the 
    instruction 'INC' and its necessary input 'A' are
    needed and henced parsed. 

**************************************************************/

/*** INCLUDE FILES ***/

#include <stdio.h>
#include <string.h>
#include <stdlib.h> 
#include <ctype.h>
#include <memory.h>

#include "cool.h"
#include "parse.h"      
#include "instruc.h"


class Instruction * Parser::GetInstruc( char *in_str )
{

int  
    status, token_cnt, 
    i, tmp_int;

char 
    *token,
    *params [ COOL_MAX_PARAMS ];

char
    token_seps[] = "(,)\n",
    str_buff[ MAX_STRING_LEN ],
    ins_buff[ COOL_MAX_INSTR_LEN + 1 ];

InstructionID      insID;     // enum
class Instruction  *instruc;


// Init

status      = SUCCESS; 
instruc     = NULL;
token_cnt   = 0;
str_buff[0] = '\0';

for( i = 0; i < COOL_MAX_PARAMS; ++i )
    params[ i ] = (char *)NULL;

memset( ins_buff, '\0', (COOL_MAX_INSTR_LEN + 1) );

strcpy( str_buff, in_str );
tmp_int = strlen(str_buff);

for( i = 0; i < tmp_int; ++i )
    str_buff[ i ] = toupper(str_buff[i]);


// Parse instruction  exs: ADD(C,4)    END 

token = strtok( str_buff, token_seps );

while( token != NULL && token_cnt <= COOL_MAX_PARAMS )
{
    // While there are tokens in "string" 

    if( token_cnt == 0 )
    {
        strncpy( ins_buff, token, COOL_MAX_INSTR_LEN );
        ++token_cnt;
    }
    else
    {
        params[ token_cnt-1 ] = strdup( token );
        ++token_cnt;
    }
    
    // Get next token
    token = strtok( NULL, token_seps );
}


// Create appropriate Instruction Object & record InstructionID

if( !(strcmp(ins_buff, COOL_INSTR_ASSIGN_STR)) )
{
    // Instruction-Sequential-Assignment
    instruc = new Assign();
    insID = (InstructionID)assign;
}
else if( !(strcmp(ins_buff, COOL_INSTR_INC_STR)) )
{
    // Instruction-Sequential-Increment

     instruc = new Increment();
     insID = (InstructionID)increment;
}
else if( !(strcmp(ins_buff, COOL_INSTR_ADD_STR)) )
{
    // Instruction-Sequential-Add

     instruc = new Add();
     insID = (InstructionID)add;
}
else if( !(strcmp(ins_buff, COOL_INSTR_SUB_STR)) )
{
    // Instruction-Sequential-Subtract

     instruc = new Subtract();
     insID = (InstructionID)subtract;
}
else if( !(strcmp(ins_buff, COOL_INSTR_MULTIPLY_STR)) )
{
    // Instruction-Sequential-Subtract

     instruc = new Multiply();
     insID = multiply;
}
else if( !(strcmp(ins_buff, COOL_INSTR_COPY_STR)) )
{
    // Instruction-Sequential-Copy

     instruc = new Copy();
     insID = (InstructionID)copy;
}
else if( !(strcmp(ins_buff, COOL_INSTR_ENDWHILE_STR)) )
{
    // Instruction-Sequential-EndSet

     instruc = new EndSet();
     insID = (InstructionID)endwhile;
}
else if( !(strcmp(ins_buff, COOL_INSTR_END_STR)) )
{
    // Instruction-Sequential-End

     instruc = new End();
     insID = (InstructionID)end;
}
else if( !(strcmp(ins_buff, COOL_INSTR_WHILENE_STR)) )
{
    // Instruction-Loop-While

     instruc = new LoopWhile();
     insID = (InstructionID)whilene;
}
else if( !(strcmp(ins_buff, COOL_INSTR_IFTHEN_STR)) )
{
    // Instruction-Conditional-IfThen

     instruc = new IfThenElse();
     insID = (InstructionID)ifthen;
}
else if( !(strcmp(ins_buff, COOL_INSTR_ELSE_STR)) )
{
    // Instruction-Conditional-Else

     instruc = new Else();
     insID = (InstructionID)ifelse;
}
else if( !(strcmp(ins_buff, COOL_INSTR_ENDIF_STR)) )
{
    // Instruction-Conditional-EndSet

     instruc = new EndSet();
     insID = (InstructionID)endconditional;
}
else
{
    status = ERROR_INVAL_INSTR;
    goto RETURN;
}


// If success so far, assign InstructionID and Instruction Params

instruc->SetInstrucID( insID  );
status = instruc->SetParams( params );


RETURN:


if( status != SUCCESS )
{
    if( status == ERROR_INVAL_INSTR )
        sprintf( message, "%s%s", ERROR_INVAL_INSTR_STR, in_str );
    else
    {
        instruc->GetMessage( message );
        sprintf( str_buff, "%s%s;%s", ERROR_INVAL_INSTR_STR, in_str, message );
        strcpy( message, str_buff );
    }

    if( instruc ) delete instruc;
    instruc = NULL;
}
 
for( i = 0; i < COOL_MAX_PARAMS; ++i )
{
    if( params[i] ) delete params[i];
}

return( instruc );
}

