// File name: cool.h
//
// Purpose:  This file will define the classes to support the cool program
//					 It also provides #define values

#ifndef cool											// test if classes are already defined 
 #define cool 
 #define TRUE       1
 #define FALSE      0

//Status values
 #define SUCCESS			  100
 #define INVALID_SYNTAX 101				// Invalid syntax
 #define INVALID_INST   102				// Invalid instruction
 #define NO_VALUE       103				// No value entered
 #define NO_INSTRUCT    104				// No instruction entered
 #define INVALID_VAR    105				// Invalid variable - not an alpha character
 #define OUT_OF_RNG		  106				// Value is too large or too small for an INT
 #define FILE_OPEN_ERR  107				// Can not open input file
 #define LOOP_IN_LOOP   108				// Repeat or Whilene nested
 #define TOO_MANY_CHAR  109				// Variable has more than 1 character
 #define UNDEFINED			110				// Variable has not been declared
	
 #define INSTRUCT_LEN     8				//Maximum number of chars in an instruction
 #define MAX_INT_LEN     11				//Maximum number of chars in an integer
 #define VAR_DECLARED			1			
 #define NOT_DECLARED			0

 // Loop values
 #define NO_LOOP		  0
 #define ADD_TO_LOOP  1
 #define PROCESS_LOOP 2
 #define DELETE_LOOP  3	
 #define SKIP_LOOP	  4	

// selection values
 #define DO_NOT_EXECUTE 0						//used in the if_then and else instructions
 #define EXECUTE        1						//used in the if_then and else instructions

 #include <string.h>
 #include <fstream.h>
 #include <iomanip.h>
 #include <stdio.h>
 #include <LIMITS.H>						//provides INT_MAX 2147483647 and INT_MIN 2147483647 

 typedef int boolean;

 // *******  class Overflow   *******
 class Overflow 						 //exception handler object for integer overflow
 {
 public:
	  Overflow (){};
	 ~Overflow (){};
 };

 // *******  class data   *******
 class data
 {
 public:
	 int variables[52];				// array of variable values
	 int declared_flg[52];			// 0 - undeclared, 1 (DECLARED) - declared variable
	 char last_char;					// last variable entered
	 data(void);
 };

// *******  class instruction   *******
 class instruction
 {
 protected:
	 int variable_index1;			//index of the variable on which to operate
	 int variable_index2;			//index of variable from which of obtain a value
	 int value;								// value input with instruction statement

 public:
	instruction(void);
 	virtual int check_syntax(char*)
		{ return SUCCESS; };
	virtual void execute(void)
		{ };
	virtual void my_id(char*);
 };

 	// *******  class sequential   *******
 class sequential : public instruction
 {
 public:
 	sequential(void);
	int check_syntax(char*);
	void execute(void)
	{  };
 };

	// *******  class selection   *******
 class selection : public instruction
 {
 protected:
	 int sel_var_index;					//index of selection variable
 public:
	 selection(void);
 };

 // *******  class loop_instruct   *******
 class loop_instruct : public instruction
 {
 public:
	 instruction *loop_instruction;
	 loop_instruct *next_instruction;
 };

 // *******  class loop   *******
 class loop : public instruction 
 {
	 loop_instruct *begin_loop;
   loop_instruct *end_loop;
	 protected:
	 int loop_variable_ind;					//index of loop variable
	 int loop_var_test_ind;					//index of the loop test variable
	 int loop_value_test;						//value to test if a variable is not entered
 public:
	 loop(void);
	 ~loop(void);
	 int check_syntax(char* string);
	 void execute(void);
	 void add_instruction(instruction*);
	 void process_loop(void);
	 void delete_loop(void);
 };

  // *******  class repeat  *******
 class repeat : public loop
 {
 public:
	int check_syntax(char*);
	void execute(void);
 };

  // *******  class until   *******
 class until : public loop
 {
 public:
	void my_id(char*);
 };

// *******  class whilene   *******
 class whilene : public loop
 {
 };

	// *******  class endwhile   *******
 class endwhile : public loop
 {
 public:
	int check_syntax(char*);
	void execute(void);
 };

	// *******  class if_then    *******
 class if_then : public selection
 {
	 int if_then_var_index;
 public:
	 short if_then_flg;
	 int check_syntax(char*);
	 void execute(void);
 	 void my_id(char*);
 };

 	// *******  class ELSE   *******
class ELSE : public selection
 {
 public:
	 int check_syntax(char*);
	 void execute(void);
 	 void my_id(char*);
 };

 	// *******  class endif   *******
class endif : public selection
 {
 public:
	 int check_syntax(char*);
	 void execute(void);
 	 void my_id(char*);
 };

 	// *******  class end   *******
class end : public instruction
 {
 public:
	int check_syntax(char*);
	void execute(void);
 };

 	// *******  class add  *******
class add : public sequential
 {
 public:
	void execute(void);  
 };

  	// *******  class mul*******
class mul : public sequential
 {
 public:
	void execute(void);  
 };

  	// *******  class inc*******
class inc : public sequential
 {
	int value;
 public:
	void execute(void);  
	int check_syntax(char*);  
 };

 	// *******  class sub  *******
 class sub : public sequential
 {
 public:
	void execute(void);  // will use the add check_syntax
 };

  	// *******  class assign *******
class assign : public sequential
 {
 public:
	void execute(void);
 };

// *******  class copy   *******
 class copy : public assign
 {
 public:
	int check_syntax(char*);  //use the execute of assign
 };

// *******  class var   *******
 class var : public instruction
 {
 public:
	int check_syntax(char*); 
	void execute(void);
 };
	
// *******  class Menu   *******
 class Menu
 {
 public:
 	 char string[80];
	 void display_error_msg(int,int);    
	 void read_input(char*);      
	 int open_file(char*, FILE**);
	 int read_file(FILE*, char*); 
 };

// *******  class interpreter   *******
 class interpreter
 {
 public:
	 int process(Menu,int&);							// process input from the screen
	 int process(Menu, char*, int&);			// process input from a file
	 int process_string(char*, boolean&);
	 int parse_instruction(char*, char*);	// 	 int parse_instruction(char*, int, airport_name*);
	 int create_instruction(char*, instruction**, boolean&, loop**);
	 int parse_variable(char*, char, char, char&);
	 int parse_value(char*, char, char, int&);
	 int set_var_index(char ,int& );
	 void remove_spaces(char*,char*,int&);
 };  

#endif

