
// File name:  interpreter.cpp
// Author:   Rebecca Barry
// Purpose:  provides the implementation of the methods of the interpreter class
#include "cool.h"
extern data vars;
extern short loop_flg;
extern short selection_flg;

//  **************** process from screen input *****************
int interpreter::process(Menu menu_1,int& line_count)
{
	char string[512];						//input string holding line of code
  boolean more_to_follow = TRUE;
	int status = SUCCESS;

	while((status == SUCCESS) && more_to_follow)
	{
		menu_1.read_input(string);
		line_count++;
		status = process_string(string, more_to_follow);
	}
	return status;
};

//  **************** process from file *****************
int interpreter::process(Menu menu_1, char* file_name, int& line_count)
{
	char string[512];						//input string holding line of code
  boolean more_to_follow = TRUE;
	int status = SUCCESS;
	FILE *input_file;

	status = menu_1.open_file(file_name, &input_file);
	while((status == SUCCESS) && more_to_follow)
	{
		menu_1.read_file(input_file,string);
		line_count++;
		status = process_string(string, more_to_follow);
	}
	return status;
};

//  **************** parse_instruction *****************
#include <ctype.h>
int interpreter::parse_instruction(char *string, char *instruct) 
{
	int status = SUCCESS;
	char *ptr1;
	int len = 0;
	int n = 0;

	instruct[0] = '\0';
	ptr1 = strchr(string, '(' );			//check for the instructions that require a (

	if (ptr1 != NULL)
	{
		n = ptr1 - string;
		if( n > 0 && n <= INSTRUCT_LEN)		//test for 0 and for too many characters
		{
			strncpy(instruct,string,n);			//copy the instruction from the string into instruct
			instruct[n] = '\0';
		}
		else
			status = INVALID_INST;
	}
	else														//check for the other instructions
	{
		len = strlen(string);
		if( len == 0 ) status = NO_INSTRUCT;   // operator did not enter anything
		else
		{
			ptr1 = strchr(string, ',' );				// check for the var instruction
			if (ptr1 != NULL)
			{
				strncpy(instruct,string,3);
				strcpy(string,(string+3));
			}
			else
			{
				if( len <= INSTRUCT_LEN )
					strcpy(instruct,string);			//  Must be ENDWHILE, END, ELSE, REPEAT, or VAR
				else status = INVALID_INST;			// instruction contains too many characters
			}
		}
	}
	if( status == SUCCESS)
	{
		len = strlen(instruct);
		for (n=0; n < len; n++)instruct[n]=toupper(instruct[n]);
	}
 	return status;
};

//  **************** parse_variable *****************
int interpreter::parse_variable(char *string, char delimeter1, 
																char delimeter2, char& variable)
{
	int status = SUCCESS;
	char *ptr1, *ptr2;
	boolean more_to_follow = TRUE;
	int n = 0;

	variable = '\0';
	ptr1 = strchr(string, delimeter1);
	ptr2 = strchr(string, delimeter2);
	if(ptr1 && ptr2)
	{
		n = ptr2 - ptr1 -1;
		if(n == 1)
		{
			variable = (ptr1+1)[0];
		}
		else
		{
			if(n > 1)status = TOO_MANY_CHAR;
			else status = INVALID_SYNTAX;
		}
	}
	else
		status = INVALID_SYNTAX;
	return status;
};

//  **************** set_var_index *****************
int interpreter::set_var_index(char var,int& index_of_var)
{
	char first_u_char = 'A';
	char last_u_char  = 'Z';
	char first_l_char = 'a';
	char last_l_char  = 'z';
	int status = SUCCESS;

	if(int(var) >= int(first_u_char) &&  int(var) <= int(last_u_char))
		index_of_var = int(var) - int(first_u_char);
	else if (int(var) >= int(first_l_char) &&  int(var) <= int(last_l_char))
		index_of_var = int(var) - int(first_l_char) + 26;
	else
		status = INVALID_VAR;   // character is not a letter
	if(status != INVALID_VAR && vars.declared_flg[index_of_var] != VAR_DECLARED)
	{
		status = UNDEFINED;
		vars.last_char = var;
	}
	return status;
};

//  **************** parse_value *****************
int interpreter::parse_value(char* string, char delimeter1,
														 char delimeter2, int& value)
{
	int status = SUCCESS;
	int n;											// number of digits in number
	int num;										// number of integers read
	char char_value[20];				//character string with value
	char *ptr1, *ptr2;
	double d_value;

	ptr1 = strchr(string, delimeter1);
	ptr2 = strchr(string, delimeter2);
	if( ptr1 && ptr2 )
	{
		n = ptr2 - ptr1 - 1;
		if(n <= MAX_INT_LEN)
		{
			strncpy(char_value, ptr1+1, n);
			if(n>0)
			{
				char_value[n] = '\0';
				num = sscanf(char_value, "%Lf", &d_value);  //read value into a double
				if((d_value >= INT_MIN) && (d_value <= INT_MAX))  //check size of value
					num = sscanf(char_value, "%d", &value);
				else status = OUT_OF_RNG;
				if(num == 0)status = NO_VALUE;
			}
			else status = NO_VALUE;
		}
		else status = OUT_OF_RNG;
	}
	else status = INVALID_SYNTAX;  
	return status;
};

//  **************** create_instruction *****************
int interpreter::create_instruction(char* instruct, instruction **ptr_instruct,
																		boolean& more_to_follow, loop **ptr_loop)																		
{
	int status = SUCCESS;

	if( strcmp(instruct,"ASSIGN") == 0)
	{
		*ptr_instruct = new assign;
	}
		else if(strcmp(instruct,"ADD")==0)
	{
		*ptr_instruct = new add;
	}
		else if(strcmp(instruct,"COPY")==0)
	{
		*ptr_instruct = new copy;
	}
	else if(strcmp(instruct,"INC")==0)
	{
		*ptr_instruct = new inc;
	}
	else if(strcmp(instruct,"SUB")==0)
	{
		*ptr_instruct = new sub;
	}
	else if(strcmp(instruct,"WHILENE")==0)
	{
		*ptr_instruct = new whilene;
		if(*ptr_loop)status = LOOP_IN_LOOP;
		else *ptr_loop = new loop;
	}
	else if(strcmp(instruct,"ENDWHILE")==0)
	{
		*ptr_instruct = new endwhile;
	}
	else if(strcmp(instruct,"REPEAT")==0)
	{
		*ptr_instruct = new repeat;
		if(*ptr_loop)status = LOOP_IN_LOOP;
		else *ptr_loop = new loop;
	}
	else if(strcmp(instruct,"UNTIL")==0)
	{
		*ptr_instruct = new until;
		loop_flg = PROCESS_LOOP;
	}
	else if(strcmp(instruct,"MUL")==0)
	{
		*ptr_instruct = new mul;
	}
	else if(strcmp(instruct,"IF")==0)
	{
		*ptr_instruct = new if_then;
	}
	else if(strcmp(instruct,"ELSE")==0)
	{
		*ptr_instruct = new ELSE;
	}
	else if(strcmp(instruct,"ENDIF")==0)
	{
		*ptr_instruct = new endif;
	}
	else if(strncmp(instruct,"VAR",3)==0)
	{
		*ptr_instruct = new var;
	}
	else if(strcmp(instruct,"END")==0)
	{
		*ptr_instruct = new end;
		more_to_follow = FALSE;
	}
	else
		status = INVALID_INST;

	return status;
};

//  **************** remove_spaces *****************
void interpreter::remove_spaces(char* string, char* ret_strng, int& len)
{
	int i=0;
	int j=0;
	
	len = strlen(string);
	for (i = 0; i < len; i++)
	{
		if(string[i] != ' ')
		{
			ret_strng[j]=string[i];
			j++;
		}
	} //end for
	len = j;
	ret_strng[j] = '\0';
};
				
//  **************** process_string method *****************
int interpreter::process_string(char* string, boolean &more_to_follow)
{
	instruction *ptr_instruct;
	char instruct[20];						//holds the instruction which at most is 8 characters
	char inst_id[20];
	int status = SUCCESS;
	static loop *ptr_loop;

	more_to_follow = TRUE;
	status = parse_instruction(string, instruct);
	if (status == SUCCESS)
	{
		status = create_instruction(instruct, &ptr_instruct,
																	more_to_follow, &ptr_loop);									
		// Now perform the syntax check and execute methods
		if(status == SUCCESS)
		{
			status = ptr_instruct->check_syntax(string);
			if(status == SUCCESS)
			{
				//The loop variables may have been equal when the WHILENE instruction was read
				// if so - the following instructions must be skipped but if the instruction is
				// else or endif - we must execute it in order to reset the selection_flg
				ptr_instruct->my_id(inst_id);
				if( (strcmp(inst_id, "endif") == 0) || (strcmp(inst_id, "ELSE") == 0) ||
				(loop_flg != SKIP_LOOP && selection_flg == EXECUTE))
						ptr_instruct->execute();
					
				//if the loop variables are equal on the first pass:
				// IF the WHILENE instruction is read,
				// the loop_flg will be set to DELETE_LOOP before a loop is created. 
				// Set the loop_flg to SKIP_FLG and skip the following instrutions until the 
				// ENDWHILE is read.
				// If it a REPEAT loop, the loop has been created and must be deleted.
				if(loop_flg == DELETE_LOOP)
				{
					if( strcmp(inst_id, "until") == 0)
					{
						ptr_loop->delete_loop();
						if(ptr_loop) delete ptr_loop;
						ptr_loop = NULL;	
					}
					else
						loop_flg = SKIP_LOOP;
						if(ptr_loop) delete ptr_loop; // pointer to loop has been created
						ptr_loop = NULL;						 //	delete and reset the pointer to NULL
				}
				if(loop_flg == ADD_TO_LOOP || loop_flg == PROCESS_LOOP)
				{
					ptr_loop->add_instruction(ptr_instruct);
					while(loop_flg == PROCESS_LOOP)
						ptr_loop->process_loop();
					if(loop_flg == DELETE_LOOP)
					{
						ptr_loop->delete_loop();
						ptr_loop = NULL;
					}
				}//end if(loop_flg)
			}//end if(status)
		}//end if(status)
	}//end if
	if(ptr_instruct)delete ptr_instruct;
	return status;
};
