
// File name:  loop.cpp
// Author:   Rebecca Barry
// Purpose:  provides the implementation of the methods of the loop class

#include "cool.h"
extern data vars;
extern short loop_flg;
extern short selection_flg;
extern interpreter interpret;

// *************** class loop methods  **********************//
//
//***************** constructor method ***********************//
loop::loop(void)
	{
		loop_variable_ind = -1;				//index of loop variable
		loop_var_test_ind = -1;				//index of the loop test variable
		loop_value_test   =  0;				//value to test if a variable is not entered
		begin_loop = NULL;
		end_loop   = NULL;
		if(loop_flg == SKIP_LOOP)
			loop_flg = NO_LOOP;
		else
			loop_flg = ADD_TO_LOOP;
	};

//***************** destructor method ***********************//
loop::~loop(void)
	{
		delete begin_loop;
		begin_loop = NULL;
		delete end_loop;
		end_loop = NULL;
	};

// Method to check the syntax of the loop instruction

int loop::check_syntax(char* string)
{
	char var;
	char delimeter1 = '(';
	char delimeter2 = ',';
	char *ptr;
	int len = 0;
	int status = SUCCESS;

	//determine the loop variable  that we will assign 
	status = interpret.parse_variable(string, delimeter1,
																		delimeter2,var);
	if(status == SUCCESS)
	{
		//determine the loop variable index of the variable
		status = interpret.set_var_index(var,loop_variable_ind);
		
		if(status == SUCCESS)
		{
		//determine the value or variable will be tested against
			delimeter1 = ',';
			delimeter2 = ')';
			status = interpret.parse_variable(string, delimeter1,
																				delimeter2,var);
			if(status == SUCCESS)
			{
				//determine the variable index of the test variable
				status = interpret.set_var_index(var,loop_var_test_ind);
			}
		}
	}
	if (status == SUCCESS)
	//  Make sure that there is no garbage after the instruction
	{
		ptr = strchr(string,')');
		len = strlen(string) - 1;
		if((ptr - string) != len)status = INVALID_SYNTAX;
	}
	return status;
}; //end of loop::check_syntax

// Method to execute the loop
void loop::execute(void)
{
	if(vars.variables[loop_variable_ind] == vars.variables[loop_var_test_ind])
		loop_flg = DELETE_LOOP;
};

// Method to add an instruction to the loop    
void loop::add_instruction(instruction *new_instruction)											
{
	loop_instruct *temp;

	if (begin_loop == NULL)            //loop is empty
	{
		begin_loop = new loop_instruct;
		end_loop = new loop_instruct;
		end_loop->loop_instruction = NULL;
		end_loop->next_instruction = NULL;
			
		begin_loop->loop_instruction = new_instruction;
		begin_loop->next_instruction = end_loop;		
	}
	else if(end_loop->loop_instruction == NULL)
	{
		 end_loop->loop_instruction = new_instruction;
	}
	 else
  {
		temp = new loop_instruct;
		temp->loop_instruction = new_instruction;
		temp->next_instruction = NULL;
		end_loop->next_instruction = temp; 
		end_loop = temp;	
  }
};

//******************** process_loop method ********************
void loop::process_loop(void)
{
	loop_instruct *temp;
	char inst_id[20];
	while(loop_flg == PROCESS_LOOP )
	{
		temp = begin_loop;
		while (temp != NULL && loop_flg == PROCESS_LOOP)
		{	
			if(selection_flg == EXECUTE)
				temp->loop_instruction->execute();
			else
			{
				temp->loop_instruction->my_id(inst_id);
				if( (strcmp(inst_id, "endif") == 0) || (strcmp(inst_id, "ELSE") == 0) )
					temp->loop_instruction->execute();
			}
			temp = temp->next_instruction;
		}
	}
}; 

//************* delete_loop method*************************
void loop::delete_loop(void)
{
	loop_instruct *temp,*temp1;

	if(begin_loop == NULL)
		loop_flg = SKIP_LOOP;
	else
	{
		temp = begin_loop;
		while (temp != NULL)
		{	
			temp1 = temp->next_instruction;
			delete temp->loop_instruction;					//delete the pointer to the instruction
			delete temp;												//delete the loop link
			temp = temp1;
		}
		loop_flg = NO_LOOP;
	}
};
