/*
 * ServerConnection
 *
 * @version 1.0
 * @author American Management Systems
 */

import java.net.*;
import java.io.*;

public class ServerConnection {
   private Socket sock;
   private BufferedInputStream in;
   private PrintStream out;
   private boolean errorFlag = false;

  /**
   * Create a connection to the server using a socket
   *
   * @param      host  Destination address
   * @param      port  Destination port
   */
   public ServerConnection(String host, String port)
   {
      try {
         errorFlag = false;
         sock = new Socket(host,new Integer(port).intValue());
         in   = new BufferedInputStream(sock.getInputStream());
         out  = new PrintStream(new BufferedOutputStream(sock.getOutputStream()), false);
      } catch (Exception e) {
         errorFlag = true;
      }
   }

  /**
   * Send a data string over the server connection
   *
   * @param      data  Data to be transmitted to the Application Server
   */
   public void send(String data)
   {
      errorFlag = false;
      try {
         out.print(data);
         out.flush();
      } catch (Exception e) {
         errorFlag = true;;
      }
   }

  /**
   * Receive data from the Application Server
   *
   * <p> Reads all data coming in on a socket until the end of stram has been
   * reached
   *
   * @return     Data string that was received
   */
   public String receive() {

      int          inCount;
      byte         bytebuffer[] = new byte[1025];
      String       lastData;
      StringBuffer dataContent = new StringBuffer();

      errorFlag = false;
      try {
         while ((inCount = in.read(bytebuffer, 0 , bytebuffer.length)) != -1) {
            lastData = new String(bytebuffer, 0, 0, inCount);
            dataContent.append(lastData);
         }
      } catch (Exception e) {
        errorFlag = true;
        return "";
      }
      return (dataContent.toString());
   }

  /**
   * Close the Application Server Connection
   */
   public void close() {
      errorFlag = false;
      try {
         out.close();
         in.close();
         sock.close();
      } catch (Exception e) {
        errorFlag = true;
      }
   }

  /**
   * Retrieve the error flag of the ServerConnection
   *
   * @return     The error flag. TRUE if an error has occurred
   */
   public boolean error() {
      return (errorFlag);
   }
}

